@@ -1,7 +1,9 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2000-2024. All Rights Reserved.
+ * SPDX-License-Identifier: Apache-2.0
+ *
+ * Copyright Ericsson AB 2000-2025. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -63,8 +65,6 @@
 
 #define ERTS_BPF_ALL              0x3FF
 
-erts_atomic32_t erts_active_bp_index;
-erts_atomic32_t erts_staging_bp_index;
 erts_mtx_t erts_dirty_bp_ix_mtx;
 
 ErtsTraceSession* erts_staging_trace_session;
@@ -141,6 +141,7 @@ static GenericBpData* check_break(ErtsTraceSession *session,
 
 static void bp_meta_unref(BpMetaTracer *bmt);
 static void bp_count_unref(BpCount *bcp);
+static BpDataCallTrace* bp_calltrace_alloc(void);
 static void bp_calltrace_unref(BpDataCallTrace *bdt);
 static void consolidate_bp_data(struct erl_module_instance *mi,
                                 ErtsCodeInfo *ci, int local);
@@ -222,12 +223,15 @@ erts_bp_match_functions(BpFunctions* f, ErtsCodeMFA *mfa, int specified)
             case 3:
                 if (ci->mfa.arity != mfa->arity)
                     continue;
+                ERTS_FALLTHROUGH();
             case 2:
                 if (ci->mfa.function != mfa->function)
                     continue;
+                ERTS_FALLTHROUGH();
             case 1:
                 if (ci->mfa.module != mfa->module)
                     continue;
+                ERTS_FALLTHROUGH();
             case 0:
                 break;
             }
@@ -260,12 +264,15 @@ erts_bp_match_export(BpFunctions* f, ErtsCodeMFA *mfa, int specified)
         case 3:
             if (mfa->arity != ep->info.mfa.arity)
                 continue;
+            ERTS_FALLTHROUGH();
         case 2:
             if (mfa->function != ep->info.mfa.function)
                 continue;
+            ERTS_FALLTHROUGH();
         case 1:
             if (mfa->module != ep->info.mfa.module)
                 continue;
+            ERTS_FALLTHROUGH();
         case 0:
             break;
         default:
@@ -489,12 +496,10 @@ consolidate_bp_data_session(GenericBp* g)
     if (flags & ERTS_BPF_TIME_TRACE) {
 	dst->time = src->time;
 	erts_refc_inc(&dst->time->refc, 1);
-	ASSERT(dst->time->hash);
     }
     if (flags & ERTS_BPF_MEM_TRACE) {
 	dst->memory = src->memory;
 	erts_refc_inc(&dst->memory->refc, 1);
-	ASSERT(dst->memory->hash);
     }
 }
 
@@ -877,6 +882,7 @@ erts_clear_all_export_break(Module* modp, Export *ep)
  */
 static void fixup_cp_before_trace(Process *c_p,
                                   Eterm cp_save[2],
+                                  ErtsTraceSession* session,
                                   int *return_to_trace)
 {
     const ErtsFrameLayout frame_layout = erts_frame_layout;
@@ -892,15 +898,16 @@ static void fixup_cp_before_trace(Process *c_p,
 
     for (;;) {
         ErtsCodePtr w;
-
-        erts_inspect_frame(cpp, &w);
+        const Eterm *frame = erts_inspect_frame(cpp, &w);
 
         if (BeamIsReturnTrace(w)) {
             cpp += CP_SIZE + BEAM_RETURN_TRACE_FRAME_SZ;
         } else if (BeamIsReturnCallAccTrace(w)) {
             cpp += CP_SIZE + BEAM_RETURN_CALL_ACC_TRACE_FRAME_SZ;
         } else if (BeamIsReturnToTrace(w)) {
-            *return_to_trace = 1;
+            if (frame[0] == session->weak_id) {
+                *return_to_trace = 1;
+            }
             cpp += CP_SIZE + BEAM_RETURN_TO_TRACE_FRAME_SZ;
         } else {
             if (frame_layout == ERTS_FRAME_LAYOUT_FP_RA) {
@@ -925,8 +932,6 @@ static void restore_cp_after_trace(Process *c_p, const Eterm cp_save[2]) {
 }
 
 static ERTS_INLINE Uint get_allocated_words(Process *c_p, Sint allocated) {
-    if (c_p->abandoned_heap)
-        return allocated + c_p->htop - c_p->heap + c_p->mbuf_sz;
     return allocated + c_p->htop - c_p->high_water + c_p->mbuf_sz;
 }
 
@@ -1108,7 +1113,7 @@ do_call_trace(Process* c_p, ErtsCodeInfo* info, Eterm* reg,
     Uint need = 0;
     Eterm* E;
 
-    fixup_cp_before_trace(c_p, cp_save, &return_to_trace);
+    fixup_cp_before_trace(c_p, cp_save, session, &return_to_trace);
 
     ERTS_UNREQ_PROC_MAIN_LOCK(c_p);
     flags = erts_call_trace(c_p, info, ms, reg, local, ref, &tracer);
@@ -1489,6 +1494,133 @@ int erts_is_call_break(Process *p, ErtsTraceSession *session, int is_time,
     return 1;
 }
 
+void erts_install_line_breakpoint(struct erl_module_instance *mi, ErtsCodePtr cp_exec) {
+    ErtsCodePtr cp_rw;
+
+    erts_unseal_module(mi);
+    cp_rw = erts_writable_code_ptr(mi, cp_exec);
+
+#ifdef BEAMASM
+    erts_asm_bp_enable(cp_rw);
+#else
+{
+    BeamInstr volatile *pc = (BeamInstr*)cp_rw;
+    BeamInstr instr = *pc;
+    BeamInstr br = BeamOpCodeAddr(op_i_enabled_line_breakpoint_t);
+
+    /* The following write is not protected by any lock.
+     * See note in erts_install_breakpoints().
+     */
+    instr = BeamSetCodeAddr(instr, br);
+    *pc = instr;
+}
+#endif
+
+    erts_seal_module(mi);
+}
+
+void erts_uninstall_line_breakpoint(struct erl_module_instance *mi, ErtsCodePtr cp_exec) {
+    ErtsCodePtr cp_rw;
+
+    erts_unseal_module(mi);
+    cp_rw = erts_writable_code_ptr(mi, cp_exec);
+
+#ifdef BEAMASM
+    erts_asm_bp_disable(cp_rw);
+#else
+{
+    BeamInstr volatile *pc = (BeamInstr*)cp_rw;
+    BeamInstr instr = *pc;
+    BeamInstr br = BeamOpCodeAddr(op_i_disabled_line_breakpoint_t);
+
+    /* The following write is not protected by any lock.
+     * See note in erts_install_breakpoints().
+     */
+    instr = BeamSetCodeAddr(instr, br);
+    *pc = instr;
+}
+#endif
+
+    erts_seal_module(mi);
+}
+
+enum erts_is_line_breakpoint erts_is_line_breakpoint_code(ErtsCodePtr p) {
+#ifdef BEAMASM
+    return beamasm_is_line_breakpoint_trampoline(p);
+#else
+    const UWord instr = *(UWord *)p;
+    if (BeamIsOpCode(instr, op_i_disabled_line_breakpoint_t))
+        return IS_DISABLED_LINE_BP;
+    if (BeamIsOpCode(instr, op_i_enabled_line_breakpoint_t))
+        return IS_ENABLED_LINE_BP;
+    return IS_NOT_LINE_BP;
+#endif
+}
+
+const Export *
+erts_line_breakpoint_hit__prepare_call(Process* c_p, ErtsCodePtr pc, Uint live, Eterm *regs, UWord *stk) {
+    FunctionInfo fi;
+    const Export *ep;
+
+    ASSERT(live <= MAX_REG);
+
+    /*
+     * Search the error_handler module
+     */
+    ep = erts_find_function(am_erts_internal, am_breakpoint, 4,
+                            erts_active_code_ix());
+    if (ep == NULL) {
+        /* No error handler */
+        return NULL;
+    }
+
+    /*
+     * Find breakpoint location
+     */
+    erts_lookup_function_info(&fi, pc, 1);
+    if (!fi.mfa) {
+        return NULL;
+    }
+
+    if (ep->info.mfa.module == fi.mfa->module
+        && ep->info.mfa.function == fi.mfa->function
+        && ep->info.mfa.arity == fi.mfa->arity) {
+        /* Cycle breaker */
+        return NULL;
+    }
+
+    /*
+     * Save live regs on the stack
+     */
+    for(int i = 0; i < live; i++) {
+        *(stk++) = regs[i];
+    }
+
+    regs[0] = fi.mfa->module;
+    regs[1] = fi.mfa->function;
+    regs[2] = make_small(fi.mfa->arity);
+    regs[3] = make_small(LOC_LINE(fi.loc));
+
+    return ep;
+}
+
+Uint
+erts_line_breakpoint_hit__cleanup(Eterm *regs, UWord *stk) {
+    int i = 0;
+
+    /*
+     * Restore X-registers
+     */
+    while(is_not_CP(*stk)) {
+        regs[i++] = *(stk++);
+    }
+
+    /*
+     * Return number of registers restored
+     */
+    return i;
+}
+
 const ErtsCodeInfo *
 erts_find_local_func(const ErtsCodeMFA *mfa) {
     const BeamCodeHeader *code_hdr;
@@ -1620,12 +1752,9 @@ static void bp_hash_delete(bp_trace_hash_t *hash) {
     hash->item = NULL;
 }
 
-static void bp_hash_reset(BpDataCallTrace* bdt) {
-    Uint i;
-    for (i = 0; i < bdt->n; i++) {
-        bp_hash_delete(&(bdt->hash[i]));
-        bp_hash_init(&(bdt->hash[i]), 32);
-    }
+static void bp_hash_reset(BpDataCallTrace** bdt_p) {
+    bp_calltrace_unref(*bdt_p);
+    *bdt_p = bp_calltrace_alloc();
 }
 
 void erts_schedule_time_break(Process *p, Uint schedule) {
@@ -1860,7 +1989,7 @@ set_function_break(ErtsCodeInfo *ci,
 	    bp->flags &= ~ERTS_BPF_TIME_TRACE_ACTIVE;
 	} else {
 	    bp->flags |= ERTS_BPF_TIME_TRACE_ACTIVE;
-	    bp_hash_reset(bp->time);
+	    bp_hash_reset(&bp->time);
 	}
 	ASSERT((bp->flags & ~ERTS_BPF_ALL) == 0);
 	return;
@@ -1869,7 +1998,7 @@ set_function_break(ErtsCodeInfo *ci,
 	    bp->flags &= ~ERTS_BPF_MEM_TRACE_ACTIVE;
 	} else {
 	    bp->flags |= ERTS_BPF_MEM_TRACE_ACTIVE;
-	    bp_hash_reset(bp->memory);
+	    bp_hash_reset(&bp->memory);
 	}
 	ASSERT((bp->flags & ~ERTS_BPF_ALL) == 0);
 	return;
@@ -1902,17 +2031,12 @@ set_function_break(ErtsCodeInfo *ci,
 	bp->count = bcp;
     } else if (break_flags & (ERTS_BPF_TIME_TRACE | ERTS_BPF_MEM_TRACE)) {
 	BpDataCallTrace* bdt;
-	Uint i;
 
 	ASSERT((break_flags & bp->flags & ERTS_BPF_TIME_TRACE) == 0);
         ASSERT((break_flags & bp->flags & ERTS_BPF_MEM_TRACE) == 0);
-	bdt = Alloc(sizeof(BpDataCallTrace));
-	erts_refc_init(&bdt->refc, 1);
-	bdt->n = erts_no_schedulers + 1;
-	bdt->hash = Alloc(sizeof(bp_trace_hash_t)*(bdt->n));
-	for (i = 0; i < bdt->n; i++) {
-	    bp_hash_init(&(bdt->hash[i]), 32);
-	}
+
+        bdt = bp_calltrace_alloc();
+
         if (break_flags & ERTS_BPF_TIME_TRACE)
             bp->time = bdt;
         else
@@ -2011,6 +2135,19 @@ bp_count_unref(BpCount* bcp)
     }
 }
 
+static BpDataCallTrace* bp_calltrace_alloc(void)
+{
+    const Uint n = erts_no_schedulers + 1;
+    BpDataCallTrace *bdt = Alloc(offsetof(BpDataCallTrace,hash) +
+                                 sizeof(bp_trace_hash_t)*n);
+    bdt->n = n;
+    erts_refc_init(&bdt->refc, 1);
+    for (Uint i = 0; i < n; i++) {
+        bp_hash_init(&(bdt->hash[i]), 32);
+    }
+    return bdt;
+}
+
 static void
 bp_calltrace_unref(BpDataCallTrace* bdt)
 {
@@ -2020,7 +2157,6 @@ bp_calltrace_unref(BpDataCallTrace* bdt)
 	for (i = 0; i < bdt->n; ++i) {
 	    bp_hash_delete(&(bdt->hash[i]));
 	}
-	Free(bdt->hash);
 	Free(bdt);
     }
 }
