@@ -1,7 +1,9 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2012-2024. All Rights Reserved.
+ * SPDX-License-Identifier: Apache-2.0
+ *
+ * Copyright Ericsson AB 2012-2025. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -62,10 +64,11 @@ typedef struct
     struct binary* match_spec;
 } ErtsTracingEvent;
 
-#ifndef ERTS_NUM_BP_IX
-//#  include "beam_bp.h"
-#  define ERTS_NUM_BP_IX 2   // ToDo FIXME UGLY
-#endif
+#define ERTS_NUM_BP_IX 2
+typedef Uint32 ErtsBpIndex;
+extern erts_atomic32_t erts_active_bp_index;
+extern erts_atomic32_t erts_staging_bp_index;
+
 struct trace_pattern_flags {
     unsigned int breakpoint : 1; /* Set if any other is set */
     unsigned int local      : 1; /* Local call trace breakpoint */
@@ -82,6 +85,15 @@ enum erts_trace_session_state {
     ERTS_TRACE_SESSION_DEAD
 };
 
+enum erts_sysmon_limit_index {
+    ERTS_SYSMON_LONG_GC = 0,
+    ERTS_SYSMON_LONG_SCHEDULE,
+    ERTS_SYSMON_LARGE_HEAP,
+    ERTS_SYSMON_LONG_MSGQ,
+
+    ERTS_SYSMON_LIMIT_CNT
+};
+
 typedef struct ErtsTraceSession {
     struct ErtsTraceSession* next;
     struct ErtsTraceSession* prev;
@@ -100,10 +112,17 @@ typedef struct ErtsTraceSession {
     struct trace_pattern_flags  on_load_trace_pattern_flags;
     ErtsTracer                  on_load_meta_tracer;
 
-    Uint32 on_spawn_proc_trace_flags;
-    ErtsTracer on_spawn_proc_tracer;
-    Uint32 on_open_port_trace_flags;
-    ErtsTracer on_open_port_tracer;
+    Uint32 new_procs_trace_flags;
+    ErtsTracer new_procs_tracer;
+    Uint32 new_ports_trace_flags;
+    ErtsTracer new_ports_tracer;
+
+    struct system_monitor_session {
+        Eterm receiver;
+        Uint limits[ERTS_SYSMON_LIMIT_CNT];
+        Sint long_msgq_off;
+        struct erts_system_monitor_flags_t flags;
+    } system_monitor;
 
 #ifdef DEBUG
     erts_refc_t dbg_bp_refc;  /* Number of breakpoints */
@@ -116,6 +135,8 @@ Eterm erts_make_trace_session_weak_ref(ErtsTraceSession*, Eterm **hpp);
 
 extern ErtsTraceSession erts_trace_session_0;
 extern erts_rwmtx_t erts_trace_session_list_lock;
+extern erts_refc_t erts_new_procs_trace_cnt;
+extern erts_refc_t erts_new_ports_trace_cnt;
 
 void erts_ref_trace_session(ErtsTraceSession*);
 void erts_deref_trace_session(ErtsTraceSession*);
@@ -140,7 +161,7 @@ void erts_assert_tracer_refs(ErtsPTabElementCommon* t_p);
 
 /* erl_bif_trace.c */
 Eterm erl_seq_trace_info(Process *p, Eterm arg1);
-void erts_system_monitor_clear(Process *c_p);
+void erts_system_monitor_clear(ErtsTraceSession*);
 void erts_system_profile_clear(Process *c_p);
 
 /* erl_trace.c */
@@ -150,18 +171,18 @@ ErtsTracer erts_set_system_seq_tracer(Process *c_p,
                                       ErtsProcLocks c_p_locks,
                                       ErtsTracer new_);
 ErtsTracer erts_get_system_seq_tracer(void);
-void erts_change_default_proc_tracing(ErtsTraceSession* session,
+void erts_change_new_procs_tracing(ErtsTraceSession* session,
                                       int setflags, Uint32 flags,
                                       const ErtsTracer tracerp);
-void erts_get_on_spawn_tracing(ErtsTraceSession*,
+void erts_get_new_proc_tracing(ErtsTraceSession*,
                                Uint32 *flagsp, ErtsTracer *tracerp);
-void erts_change_default_port_tracing(ErtsTraceSession* session,
+void erts_change_new_ports_tracing(ErtsTraceSession* session,
                                       int setflags, Uint32 flags,
                                       const ErtsTracer tracerp);
-void erts_get_on_open_port_tracing(ErtsTraceSession*,
+void erts_get_new_port_tracing(ErtsTraceSession*,
                                    Uint32 *flagsp, ErtsTracer *tracerp);
-void erts_set_system_monitor(Eterm monitor);
-Eterm erts_get_system_monitor(void);
+void erts_set_system_monitor(ErtsTraceSession*, Eterm monitor);
+Eterm erts_get_system_monitor(ErtsTraceSession*);
 int erts_is_tracer_valid(Process* p);
 
 void erts_check_my_tracer_proc(Process *);
@@ -184,12 +205,12 @@ void erts_trace_return(Process* p, ErtsCodeMFA *mfa, Eterm retval,
                        ErtsTracer tracer, Eterm session_weak_id);
 void erts_trace_exception(Process* p, ErtsCodeMFA *mfa, Eterm class_, Eterm value,
                           ErtsTracer tracer, Eterm session_weak_id);
-void erts_trace_return_to(Process *p, ErtsCodePtr pc, Eterm session_weak_id);
+void erts_trace_return_to(Process *p, ErtsCodePtr pc, ErtsTracerRef *ref);
 void trace_sched(Process*, ErtsProcLocks, Eterm, Uint32 trace_flag);
 void trace_sched_session(Process*, ErtsProcLocks, Eterm what, ErtsTracerRef*);
 void trace_proc(Process*, ErtsProcLocks, Process*, Eterm, Eterm);
 void trace_proc_spawn(Process*, Eterm what, Eterm pid, Eterm mod, Eterm func, Eterm args);
-void save_calls(Process *p, Export *);
+void save_calls(Process *p, const Export *);
 void trace_gc(Process *p, Eterm what, Uint size, Eterm msg);
 /* port tracing */
 void trace_virtual_sched(Process*, ErtsProcLocks, Eterm);
@@ -215,8 +236,15 @@ void monitor_long_gc(Process *p, Uint time);
 void monitor_long_schedule_proc(Process *p, const ErtsCodeMFA *in_i,
                                 const ErtsCodeMFA *out_i, Uint time);
 void monitor_long_schedule_port(Port *pp, ErtsPortTaskType type, Uint time);
-void monitor_large_heap(Process *p);
-void monitor_generic(Process *p, Eterm type, Eterm spec);
+void monitor_large_heap(Process *p, Uint size);
+void monitor_busy_port(Process *p, Eterm spec);
+void monitor_busy_dist_port(Process *p, Eterm spec);
+void monitor_long_msgq_on(Process*);
+void monitor_long_msgq_off(Process*);
+void erts_clear_all_msgq_low_sessions(Process *p);
+void erts_consolidate_all_msgq_low_sessions(Process *p);
+
+
 Uint erts_trace_flag2bit(Eterm flag);
 int erts_trace_flags(ErtsTraceSession*, Eterm List,
                      Uint *pMask, ErtsTracer *pTracer, int *pCpuTimestamp);
@@ -273,6 +301,7 @@ void erts_tracer_replace(ErtsPTabElementCommon *t_p,
                          const ErtsTracer new_tracer);
 void erts_tracer_update_impl(ErtsTracer *tracer, const ErtsTracer new_tracer);
 int erts_tracer_nif_clear(void);
+bool erts_get_tracer_pid(ErtsTracer, Eterm* pid);
 
 #define erts_tracer_update(t,n) do { if (*(t) != (n)) erts_tracer_update_impl(t,n); } while(0)
 #define ERTS_TRACER_CLEAR(t) erts_tracer_update(t, erts_tracer_nil)
@@ -291,6 +320,7 @@ ERTS_DECLARE_DUMMY(erts_tracer_nil) = NIL;
 #define IS_TRACER_VALID(tracer)                                         \
     (ERTS_TRACER_COMPARE(tracer,erts_tracer_true)                       \
      || ERTS_TRACER_IS_NIL(tracer)                                      \
+     || is_internal_pid(tracer)                                         \
      || (is_list(tracer) && is_atom(CAR(list_val(tracer)))))
 
 #define ERTS_TRACER_FROM_ETERM(termp) \
