@@ -1,3 +1,23 @@
+%% %CopyrightBegin%
+%%
+%% SPDX-License-Identifier: Apache-2.0
+%%
+%% Copyright 2004-2010 held by the authors. All Rights Reserved.
+%% Copyright Ericsson AB 2023-2025. All Rights Reserved.
+%%
+%% Licensed under the Apache License, Version 2.0 (the "License");
+%% you may not use this file except in compliance with the License.
+%% You may obtain a copy of the License at
+%%
+%%     http://www.apache.org/licenses/LICENSE-2.0
+%%
+%% Unless required by applicable law or agreed to in writing, software
+%% distributed under the License is distributed on an "AS IS" BASIS,
+%% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+%% See the License for the specific language governing permissions and
+%% limitations under the License.
+%%
+%% %CopyrightEnd%
 -module(cplt_SUITE).
 
 -include_lib("common_test/include/ct.hrl").
@@ -235,8 +255,7 @@ update_plt(Config) ->
     ErlangBeam = case code:where_is_file("erlang.beam") of
                      non_existing ->
                          filename:join([code:root_dir(),
-                                        "erts", "preloaded", "ebin",
-                                        "erlang.beam"]);
+                                        "erts", "ebin", "erlang.beam"]);
                      EBeam ->
                          EBeam
                  end,
@@ -289,8 +308,7 @@ local_fun_same_as_callback(Config) when is_list(Config) ->
     ErlangBeam = case code:where_is_file("erlang.beam") of
                      non_existing ->
                          filename:join([code:root_dir(),
-                                        "erts", "preloaded", "ebin",
-                                        "erlang.beam"]);
+                                        "erts", "ebin", "erlang.beam"]);
                      EBeam ->
                          EBeam
                  end,
@@ -901,8 +919,7 @@ erlang_beam() ->
     case code:where_is_file("erlang.beam") of
         non_existing ->
             filename:join([code:root_dir(),
-                           "erts", "preloaded", "ebin",
-                           "erlang.beam"]);
+                           "erts", "ebin", "erlang.beam"]);
         EBeam ->
             EBeam
     end.
