@@ -1,5 +1,7 @@
 %%
 %% %CopyrightBegin%
+%%
+%% SPDX-License-Identifier: Apache-2.0
 %% 
 %% Copyright Ericsson AB 2018-2025. All Rights Reserved.
 %% 
@@ -97,7 +99,10 @@
       Init :: initial_evaluator_state().
                              
 start(Name, Seq, InitState) 
-  when is_list(Name) andalso is_list(Seq) andalso (Seq =/= []) ->
+  when is_list(Name) andalso
+       is_list(Seq)  andalso
+       (Seq =/= [])  andalso
+       is_map(InitState) ->
     %% Make sure 'parent' is not already used
     case maps:find(parent, InitState) of
         {ok, _} ->
@@ -171,6 +176,16 @@ await_finish([], _OK, Fails) ->
     Fails;
 await_finish(Evs, OK, Fails) ->
     receive
+        {'EXIT', _Pid, {timetrap_timeout, _Timeout, _Stack}} ->
+            %% The test timeout is up.
+            ?SEV_EPRINT("timetrap timeout when: "
+                        "~n   Num Remaining Evs: ~w"
+                        "~n   OK Evs:            ~p"
+                        "~n   Failed Evs:        ~p",
+                        [length(Evs), OK, Fails]),
+            force_evs_kill(Evs),
+            exit(timetrap_timeout);
+
         %% Successful termination of evaluator
         {'DOWN', _MRef, process, Pid, normal} ->
             {Evs2, OK2, Fails2} = await_finish_normal(Pid, Evs, OK, Fails),
@@ -199,14 +214,6 @@ await_finish(Evs, OK, Fails) ->
             {Evs2, OK2, Fails2} =
                 await_finish_fail(Pid, Reason, Evs, OK, Fails),
             await_finish(Evs2, OK2, Fails2);
-
-	%% Special case: TimeTrap
-        {'EXIT', Pid, {timetrap_timeout, _TO, CallStack} = Reason} ->
-            ?SEV_IPRINT("await_finish -> timetrap (from ~p): "
-                         "~n   ~p", [Pid, CallStack]),
-	    %% force_evc_termination(Evs),
-	    {error, Reason};
-
         {'EXIT', Pid, Reason} ->
             %% ?SEV_IPRINT("await_finish -> fail (exit) received: "
             %%             "~n   Pid:    ~p"
@@ -260,15 +267,6 @@ await_finish_skip(Pid, Reason, Evs, OK) ->
     ?SEV_IPRINT("issue skip"),
     ?LIB:skip(Reason).
 
-%% force_evc_termination(Evs) ->
-%%     Kill = fun(#ev{name = Name, pid = Pid}) ->
-%% 		  ?SEV_EPRINT("kill evaluator ~p (~p) - timetrap",
-%% 			      [Name, Pid]),
-%% 		   exit(Pid, kill)
-%% 	   end,
-%%     lists:foreach(Kill, Evs).
-
-
 await_evs_terminated(Evs) ->
     Instructions =
         [
@@ -579,10 +577,10 @@ await_termination(Pid, ExpReason) ->
       Reason       :: term().
 
 await(ExpPid, Name, Announcement, Slogan, OtherPids) 
-  when (is_pid(ExpPid) orelse (ExpPid =:= any)) andalso 
-       is_atom(Name) andalso 
-       is_atom(Announcement) andalso 
-       is_atom(Slogan) andalso 
+  when (is_pid(ExpPid) orelse (ExpPid =:= any)) andalso
+       is_atom(Name) andalso
+       is_atom(Announcement) andalso
+       is_atom(Slogan) andalso
        is_list(OtherPids) ->
     receive
         skip ->
@@ -656,6 +654,22 @@ check_down(Pid, DownReason, Pids) ->
     end.
 
 
+%% ============================================================================
+
+force_evs_kill(Evs) when is_list(Evs) ->
+    force_evs_exit(Evs, kill).
+
+force_evs_exit([], _) ->
+    ok;
+force_evs_exit([#ev{name = Name,
+                    pid  = Pid,
+                    mref = MRef} | Evs], Reason) ->
+    ?SEV_IPRINT("Force terminate evaluator ~p (~p)", [Name, Pid]),
+    (catch erlang:demonitor(MRef, [flush])),
+    exit(Pid, Reason),
+    force_evs_exit(Evs, Reason).
+
+
 %% ============================================================================
 
 f(F, A) ->
@@ -678,7 +692,7 @@ print(Prefix, F, A) ->
                 %% or a named process. Instead its 
                 %% most likely the test case itself, 
                 %% so skip the name and the pid.
-                f("[~p]", [self()]);
+                "";
             SName ->
                 f("[~s][~p]", [SName, self()])
         end,
