@@ -1,6 +1,8 @@
 %%
 %% %CopyrightBegin%
 %%
+%% SPDX-License-Identifier: Apache-2.0
+%%
 %% Copyright Ericsson AB 2008-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
@@ -24,7 +26,6 @@
 -include("ssh_connect.hrl").
 -include("ssh_test_lib.hrl").
 -include_lib("common_test/include/ct.hrl").
--include_lib("stdlib/include/assert.hrl").
 
 -export([
          suite/0,
@@ -235,14 +236,135 @@ end_per_group(_, Config) ->
 
 %%--------------------------------------------------------------------
 init_per_testcase(_TestCase, Config) ->
-    %% To make sure we start clean as it is not certain that
-    %% end_per_testcase will be run!
-    end_per_testcase(any, Config),
+    ssh:stop(),
     ssh:start(),
-    ssh_test_lib:verify_sanity_check(Config).
+    {ok, TestLogHandlerRef} = ssh_test_lib:add_log_handler(),
+    ssh_test_lib:verify_sanity_check(Config),
+    [{log_handler_ref, TestLogHandlerRef} | Config].
 
-end_per_testcase(_TestCase, _Config) ->
-    ssh:stop().
+end_per_testcase(TestCase, Config) ->
+    {ok, Events} = ssh_test_lib:get_log_events(
+                     proplists:get_value(log_handler_ref, Config)),
+    EventCnt = length(Events),
+    {ok, InterestingEventCnt} = analyze_events(Events, EventCnt),
+    VerificationResult = verify_events(TestCase, InterestingEventCnt),
+    ssh_test_lib:rm_log_handler(),
+    ssh:stop(),
+    VerificationResult.
+
+analyze_events(_, 0) ->
+    {ok, 0};
+analyze_events(Events, EventNumber) when EventNumber > 0 ->
+    {ok, Cnt} = print_interesting_events(Events, 0),
+    case Cnt > 0 of
+        true ->
+            ct:comment("(logger stats) interesting: ~p boring: ~p",
+                       [Cnt, EventNumber - Cnt]);
+        _ ->
+            ct:comment("(logger stats) boring: ~p",
+                       [length(Events)])
+    end,
+    AllEventsSummary = lists:flatten([process_event(E) || E <- Events]),
+    ct:log("~nTotal logger events: ~p~nAll events:~n~s", [EventNumber, AllEventsSummary]),
+    {ok, Cnt}.
+
+process_event(#{msg := {report,
+                        #{label := Label,
+                          report := [{supervisor, Supervisor},
+                                     {Status, Properties}]}},
+                level := Level}) ->
+    format_event1(Label, Supervisor, Status, Properties, Level);
+process_event(#{msg := {report,
+                        #{label := Label,
+                          report := [{supervisor, Supervisor},
+                                     {errorContext, _ErrorContext},
+                                     {reason, {Status, _ReasonDetails}},
+                                     {offender, Properties}]}},
+                level := Level}) ->
+    format_event1(Label, Supervisor, Status, Properties, Level);
+process_event(#{msg := {report,
+                        #{label := Label,
+                          report := [{supervisor, Supervisor},
+                                     {errorContext, _ErrorContext},
+                                     {reason, Status},
+                                     {offender, Properties}]}},
+                level := Level}) ->
+    format_event1(Label, Supervisor, Status, Properties, Level);
+process_event(#{msg := {report,
+                        #{label := Label,
+                          report := [Properties, []]}},
+                level := Level}) ->
+    {status, Status} = get_value(status, Properties),
+    {pid, Pid} = get_value(pid, Properties),
+    Id = get_value(registered_name, Properties),
+    {initial_call, {M, F, Args}} = get_value(initial_call, Properties),
+    io_lib:format("[~44s]  ~6s ~30s ~20s  ~30s ~20s:~10s(~40s)~n",
+                  [io_lib:format("~p", [E]) ||
+                      E <- [Pid, Level, Label, Status, Id, M, F, Args]]);
+process_event(#{msg := {report,
+                        #{label := Label,
+                          name := Pid,
+                          reason := {Reason, _Stack = [{M, F, Args, Location} | _]}}},
+                level := Level}) ->
+    io_lib:format("[~44s]  ~6s ~30s ~20s  ~30s ~20s:~10s(~40s) ~30s~n",
+                  [io_lib:format("~p", [E]) ||
+                      E <- [Pid, Level, Label, Reason, undefined, M, F, Args, Location]]);
+process_event(#{msg := {report,
+                        #{label := Label,
+                         format := Format,
+                         args := Args}},
+                meta := #{pid := Pid},
+                level := Level}) ->
+    io_lib:format("[~44s]  ~6s ~30s ~150s~n",
+                  [io_lib:format("~p", [E]) ||
+                      E <- [Pid, Level, Label]] ++ [io_lib:format(Format, Args)]);
+process_event(E) ->
+    io_lib:format("~n||RAW event||~n~p~n", [E]).
+
+format_event1(Label, Supervisor, Status, Properties, Level) ->
+    {pid, Pid} = get_value(pid, Properties),
+    Id = get_value(id, Properties),
+    {M, F, Args} = get_mfa_value(Properties),
+    RestartType = get_value(restart_type, Properties),
+    Significant = get_value(significant, Properties),
+    io_lib:format("[~30s <- ~10s]  ~6s ~30s ~20s  ~30s ~20s:~10s(~40s) ~20s ~25s~n",
+                  [io_lib:format("~p", [E]) ||
+                      E <- [Supervisor, Pid, Level, Label, Status, Id, M, F, Args,
+                            Significant, RestartType]]).
+
+get_mfa_value(Properties) ->
+    case get_value(mfargs, Properties) of
+        {mfargs, MFA} ->
+            MFA;
+        false ->
+            {mfa, MFA} = get_value(mfa, Properties),
+            MFA
+    end.
+
+get_value(Key, List) ->
+    case lists:keyfind(Key, 1, List) of
+        R = false ->
+            ct:log("Key ~p not found in~n~p", [Key, List]),
+            R;
+        R -> R
+    end.
+
+print_interesting_events([], Cnt) ->
+    {ok, Cnt};
+print_interesting_events([#{level := Level} = Event | Tail], Cnt)
+  when Level /= info, Level /= notice ->
+    ct:log("------------~nInteresting event found:~n~p~n==========~n", [Event]),
+    print_interesting_events(Tail, Cnt + 1);
+print_interesting_events([_|Tail], Cnt) ->
+    print_interesting_events(Tail, Cnt).
+
+verify_events(_TestCase, 0) -> ok;
+verify_events(no_sensitive_leak, 1) -> ok;
+verify_events(max_channels_option, 3) -> ok;
+verify_events(_TestCase, EventNumber) when EventNumber > 0->
+    {fail, lists:flatten(
+             io_lib:format("unexpected event cnt: ~s",
+                           [integer_to_list(EventNumber)]))}.
 
 %%--------------------------------------------------------------------
 %% Test Cases --------------------------------------------------------
@@ -807,12 +929,11 @@ new_shell_xterm_term(Config) when is_list(Config) ->
                        exp_output =>
                            [<<"Enter command\r\n">>,
                             <<"1> ">>,
-                            <<"one_atom_please.\r\n\e[1022D\e[1B">>,
+                            <<"one_atom_please.\r\n">>,
                             <<"{simple_eval,one_atom_please}\r\n">>,
                             <<"2> ">>,
-                            <<"\e[3D\e[J">>,
                             <<"\e[;1;4msearch:\e[0m ">>,
-                            <<"\r\n  one_atom_please.">>]},
+                            <<"\b\b\b\b\b\b\b\b\e[J\e[;1;4msearch:\e[0m \r\n  one_atom_please.\e[A\b\b\b\b\b\b\b\b\b\b">>]},
                     Config).
 
 new_shell_helper(#{term := Term, cmds := Cmds,
@@ -883,7 +1004,7 @@ start_shell_exec(Config) when is_list(Config) ->
     {Pid, Host, Port} = ssh_test_lib:daemon([{system_dir, SysDir},
 					     {user_dir, UserDir},
 					     {password, "morot"},
-					     {exec, {?MODULE,ssh_exec_echo,[]}} ]),
+					     {exec, {?MODULE,ssh_exec_echo,["foo"]}} ]),
 
     ConnectionRef = ssh_test_lib:connect(Host, Port, [{silently_accept_hosts, true},
 						      {user, "foo"},
@@ -987,7 +1108,7 @@ start_shell_exec_fun(Config) ->
                                                   io:format("echo ~s\n", [Cmd])
                                           end)
                             end,
-                            "testing", <<"echo testing\n">>, 0,
+                            "testing", <<"echo testing\r\n">>, 0,
                             Config).
 
 start_shell_exec_fun2(Config) ->
@@ -996,7 +1117,7 @@ start_shell_exec_fun2(Config) ->
                                                   io:format("echo ~s ~s\n",[User,Cmd])
                                           end)
                             end,
-                            "testing", <<"echo foo testing\n">>, 0,
+                            "testing", <<"echo foo testing\r\n">>, 0,
                             Config).
 
 start_shell_exec_fun3(Config) ->
@@ -1005,7 +1126,7 @@ start_shell_exec_fun3(Config) ->
                                                   io:format("echo ~s ~s\n",[User,Cmd])
                                           end)
                             end,
-                            "testing", <<"echo foo testing\n">>, 0,
+                            "testing", <<"echo foo testing\r\n">>, 0,
                             Config).
 
 start_shell_exec_direct_fun(Config) ->
@@ -1070,11 +1191,11 @@ start_exec_direct_fun1_read_write(Config) ->
     {ok, Ch} = ssh_connection:session_channel(C, infinity),
 
     success = ssh_connection:exec(C, Ch, "> ", infinity),
-    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"Tiny read/write test\n">>}}),
+    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"Tiny read/write test\r\n">>}}),
 
     ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"1> ">>}}),
     ok = ssh_connection:send(C, Ch, "hej.\n", 5000),
-    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"{simple_eval,hej}\n">>}}),
+    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"{simple_eval,hej}\r\n">>}}),
 
     ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"2> ">>}}),
     ok = ssh_connection:send(C, Ch, "quit.\n", 5000),
@@ -1118,18 +1239,18 @@ start_exec_direct_fun1_read_write_advanced(Config) ->
     {ok, Ch} = ssh_connection:session_channel(C, infinity),
 
     success = ssh_connection:exec(C, Ch, "> ", infinity),
-    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"Tiny read/write test\n">>}}),
+    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"Tiny read/write test\r\n">>}}),
 
     ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"1> ">>}}),
     ok = ssh_connection:send(C, Ch, "hej.\n", 5000),
-    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"{simple_eval,hej}\n">>}}),
+    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"{simple_eval,hej}\r\n">>}}),
 
     ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"2> ">>}}),
     ok = ssh_connection:send(C, Ch, "'Hi ", 5000),
     ok = ssh_connection:send(C, Ch, "there", 5000),
     ok = ssh_connection:send(C, Ch, "'", 5000),
     ok = ssh_connection:send(C, Ch, ".\n", 5000),
-    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"{simple_eval,'Hi there'}\n">>}}),
+    ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"{simple_eval,'Hi there'}\r\n">>}}),
     ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,0,<<"3> ">>}}),
     ok = ssh_connection:send(C, Ch, "bad_input.\n", 5000),
     ssh_test_lib:receive_exec_result_or_fail({ssh_cm,C,{data,Ch,1,<<"**Error** {bad_input,3}">>}}),
@@ -1146,8 +1267,8 @@ start_exec_direct_fun1_read_write_advanced(Config) ->
     after 5000 -> go_on
     end,
     receive
-        X -> ct:fail("remaining messages"),
-             ct:log("remaining message: ~p",[X])
+        X -> ct:log("remaining message: ~p",[X]),
+        ct:fail("remaining messages")
     after 0 -> go_on
     end,
     ssh:stop_daemon(Pid).
@@ -1297,7 +1418,7 @@ start_shell_sock_exec_fun(Config) when is_list(Config) ->
 				  "testing", infinity),
 
     receive
-	{ssh_cm, ConnectionRef, {data, _ChannelId, 0, <<"echo testing\n">>}} ->
+	{ssh_cm, ConnectionRef, {data, _ChannelId, 0, <<"echo testing\r\n">>}} ->
 	    ok
     after 5000 ->
 	    ct:fail("Exec Timeout")
@@ -1342,7 +1463,7 @@ start_shell_sock_daemon_exec(Config) ->
 				  "testing", infinity),
 
     receive
-	{ssh_cm, ConnectionRef, {data, _ChannelId, 0, <<"echo testing\n">>}} ->
+	{ssh_cm, ConnectionRef, {data, _ChannelId, 0, <<"echo testing\r\n">>}} ->
 	    ok
     after 5000 ->
 	    ct:fail("Exec Timeout")
@@ -1402,7 +1523,7 @@ start_shell_sock_daemon_exec_multi(Config) ->
                             success = ssh_connection:exec(ConnectionRef, ChannelId0, "testing", infinity),
                             ct:log("~p:~p: exec on connection ~p", [?MODULE,?LINE,ConnectionRef]),
                             receive
-                                {ssh_cm, ConnectionRef, {data, _ChannelId, 0, <<"echo testing\n">>}} ->
+                                {ssh_cm, ConnectionRef, {data, _ChannelId, 0, <<"echo testing\r\n">>}} ->
                                     Parent ! {answer_received,self()},
                                     ct:log("~p:~p: received result on connection ~p", [?MODULE,?LINE,ConnectionRef])
                             after 5000 ->
@@ -1513,8 +1634,6 @@ gracefull_invalid_long_start_no_nl(Config) when is_list(Config) ->
     end.
 
 kex_error(Config) ->
-    #{level := Level} = logger:get_primary_config(),
-    ok = logger:set_primary_config(level, debug),
     PrivDir = proplists:get_value(priv_dir, Config),
     UserDir = filename:join(PrivDir, nopubkey), % to make sure we don't use public-key-auth
     file:make_dir(UserDir),
@@ -1535,10 +1654,6 @@ kex_error(Config) ->
                                    ok % Other msg
                            end,
                            self()),
-    Cleanup = fun() ->
-                      ok = logger:remove_handler(kex_error),
-                      ok = logger:set_primary_config(level, Level)
-              end,
     try
         ssh_test_lib:connect(Host, Port, [{silently_accept_hosts, true},
                                           {user, "foo"},
@@ -1556,7 +1671,7 @@ kex_error(Config) ->
             %% ok
             receive
                 {Ref, ErrMsgTxt} ->
-                    Cleanup(),
+                    ok = logger:remove_handler(kex_error),
                     ct:log("ErrMsgTxt = ~n~s", [ErrMsgTxt]),
                     Lines = lists:map(fun string:trim/1, string:tokens(ErrMsgTxt, "\n")),
                     OK = (lists:all(fun(S) -> lists:member(S,Lines) end,
@@ -1574,12 +1689,12 @@ kex_error(Config) ->
                             ct:fail("unexpected error text msg", [])
                     end
             after 20000 ->
-                    Cleanup(),
+                    ok = logger:remove_handler(kex_error),
                     ct:fail("timeout", [])
             end;
 
         error:{badmatch,{error,_}} ->
-            Cleanup(),
+            ok = logger:remove_handler(kex_error),
             ct:fail("unexpected error msg", [])
     end.
 
@@ -1617,7 +1732,7 @@ stop_listener(Config) when is_list(Config) ->
 				  "testing", infinity),
     receive
 	{ssh_cm, ConnectionRef0,
-         {data, ChannelId0, 0, <<"echo testing\n">>}} ->
+         {data, ChannelId0, 0, <<"echo testing\r\n">>}} ->
 	    ok
     after 5000 ->
 	    ct:fail("Exec Timeout")
@@ -2115,12 +2230,14 @@ test_exec_is_enabled(ConnectionRef, Exec, Expect) ->
     success = ssh_connection:exec(ConnectionRef, ChannelId, Exec, infinity),
     ExpSz = size(Expect),
     receive
-        {ssh_cm, ConnectionRef, {data, ChannelId, 0, <<Expect:ExpSz/binary, _/binary>>}} = R ->
+        {ssh_cm, ConnectionRef, {data, ChannelId, 0,
+                                 <<Expect:ExpSz/binary, _/binary>>}} = R ->
             ct:log("~p:~p Got expected ~p",[?MODULE,?LINE,R]);
         Other ->
-            %% FIXME - should this testcase fail when unexpected data is received?
             ct:log("~p:~p Got unexpected ~p~nExpect: ~p~n",
-                   [?MODULE,?LINE, Other, {ssh_cm, ConnectionRef, {data, ChannelId, 0, Expect}} ])
+                   [?MODULE,?LINE, Other, {ssh_cm, ConnectionRef,
+                                           {data, ChannelId, 0, Expect}}]),
+            {fail, "Unexpected data"}
     after 5000 ->
             {fail,"Exec Timeout"}
     end.
@@ -2202,7 +2319,7 @@ ssh_exec_echo(Cmd, User) ->
     spawn(fun() ->
                   io:format("echo ~s ~s\n",[User,Cmd])
           end).
-%% FIXME - upon refactoring this test suite, check if function below is reduntant to collect_data
+
 receive_bytes(_, _, 0, _) ->
     ct:log("ALL DATA RECEIVED Budget = 0"),
     ct:log("================================ ExpectBudget = 0 (reception completed)"),
