@@ -1,7 +1,9 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2019-2024. All Rights Reserved.
+%% SPDX-License-Identifier: Apache-2.0
+%%
+%% Copyright Ericsson AB 2019-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -25,7 +27,7 @@
 
 -include_lib("common_test/include/ct.hrl").
 -include_lib("public_key/include/public_key.hrl").
--include("ssl_record.hrl").
+-include_lib("ssl/src/ssl_record.hrl").
 
 %% Common test
 -export([all/0,
@@ -93,6 +95,16 @@
          extended_key_usage_auth/1,
          extended_key_usage_client_auth/0,
          extended_key_usage_client_auth/1,
+         extended_key_usage_mixup_client/0,
+         extended_key_usage_mixup_client/1,
+         extended_key_usage_mixup_server/0,
+         extended_key_usage_mixup_server/1,
+         extended_key_usage_ca/0,
+         extended_key_usage_ca/1,
+         extended_key_usage_ca_invalid/0,
+         extended_key_usage_ca_invalid/1,
+         extended_key_usage_ca_any/0,
+         extended_key_usage_ca_any/1,
          cert_expired/0,
          cert_expired/1,
          no_auth_key_identifier_ext/0,
@@ -131,6 +143,8 @@
          signature_algorithms_bad_curve_secp521r1/1,
          server_certificate_authorities_disabled/0,
          server_certificate_authorities_disabled/1,
+         legacy_server_certificate_authorities_disabled/0,
+         legacy_server_certificate_authorities_disabled/1,
          cert_auth_in_first_ca/0,
          cert_auth_in_first_ca/1
          ]).
@@ -157,20 +171,20 @@ groups() ->
      {'tlsv1', [], ssl_protocol_groups()},
      {'dtlsv1.2', [], tls_1_2_protocol_groups()},
      {'dtlsv1', [], ssl_protocol_groups()},
-     {rsa, [], all_version_tests() ++ rsa_tests() ++ pre_tls_1_3_rsa_tests() ++ [client_auth_seelfsigned_peer]},
-     {ecdsa, [], all_version_tests()},
-     {dsa, [], all_version_tests()},
-     {rsa_1_3, [], all_version_tests() ++ rsa_tests() ++
+     {rsa, [parallel], all_version_tests() ++ rsa_tests() ++ pre_tls_1_3_rsa_tests() ++ [client_auth_seelfsigned_peer]},
+     {ecdsa, [parallel], all_version_tests()},
+     {dsa, [parallel], all_version_tests()},
+     {rsa_1_3, [parallel], all_version_tests() ++ rsa_tests() ++
           tls_1_3_tests() ++ tls_1_3_rsa_tests() ++ [client_auth_seelfsigned_peer, basic_rsa_1024]},
-     {rsa_pss_rsae, [], all_version_tests() ++ tls_1_2_rsa_tests()},
-     {rsa_pss_rsae_1_3, [], all_version_tests() ++ rsa_tests() ++ tls_1_3_tests() ++ tls_1_3_rsa_tests()},
-     {rsa_pss_pss, [], all_version_tests()},
-     {rsa_pss_pss_1_3, [], all_version_tests() ++ rsa_tests() ++ tls_1_3_tests() ++ tls_1_3_rsa_tests()},
-     {ecdsa_1_3, [], all_version_tests() ++ tls_1_3_tests() ++
+     {rsa_pss_rsae, [parallel], all_version_tests() ++ tls_1_2_rsa_tests()},
+     {rsa_pss_rsae_1_3, [parallel], all_version_tests() ++ rsa_tests() ++ tls_1_3_tests() ++ tls_1_3_rsa_tests()},
+     {rsa_pss_pss, [parallel], all_version_tests()},
+     {rsa_pss_pss_1_3, [parallel], all_version_tests() ++ rsa_tests() ++ tls_1_3_tests() ++ tls_1_3_rsa_tests()},
+     {ecdsa_1_3, [parallel], all_version_tests() ++ tls_1_3_tests() ++
           [signature_algorithms_bad_curve_secp256r1,
            signature_algorithms_bad_curve_secp384r1,
            signature_algorithms_bad_curve_secp521r1]},
-     {eddsa_1_3, [], all_version_tests() ++ tls_1_3_tests()}
+     {eddsa_1_3, [parallel], all_version_tests() ++ tls_1_3_tests()}
     ].
 
 ssl_protocol_groups() ->
@@ -207,7 +221,8 @@ tls_1_3_tests() ->
 
 pre_tls_1_3_rsa_tests() ->
     [
-     key_auth_ext_sign_only
+     key_auth_ext_sign_only,
+     legacy_server_certificate_authorities_disabled
     ].
 
 rsa_tests() ->
@@ -256,14 +271,19 @@ all_version_tests() ->
      critical_extension_no_auth,
      extended_key_usage_auth,
      extended_key_usage_client_auth,
+     extended_key_usage_mixup_client,
+     extended_key_usage_mixup_server,
+     extended_key_usage_ca,
+     extended_key_usage_ca_invalid,
+     extended_key_usage_ca_any,
      cert_expired,
      no_auth_key_identifier_ext,
      no_auth_key_identifier_ext_keyEncipherment
     ].
 
 init_per_suite(Config) ->
-    catch crypto:stop(),
-    try crypto:start() of
+    catch application:stop(crypto),
+    try application:start(crypto) of
 	ok ->
             Config
     catch _:_ ->
@@ -292,15 +312,15 @@ do_init_per_group(Group, Config0) when Group == rsa;
                                        Group == rsa_1_3 ->
     Config1 = ssl_test_lib:make_rsa_cert(Config0),
     Config = ssl_test_lib:make_rsa_1024_cert(Config1),
-    COpts = proplists:get_value(client_rsa_verify_opts, Config),
-    SOpts = proplists:get_value(server_rsa_opts, Config),
+    COpts = ssl_test_lib:ssl_options(client_rsa_verify_opts, Config),
+    SOpts = ssl_test_lib:ssl_options(server_rsa_opts, Config),
     Version = proplists:get_value(version, Config),
     [{cert_key_alg, rsa},
      {extra_client, ssl_test_lib:sig_algs(rsa, Version)},
      {extra_server, ssl_test_lib:sig_algs(rsa, Version)} |
      lists:delete(cert_key_alg,
-                  [{client_cert_opts, COpts},
-                   {server_cert_opts, SOpts} |
+                  [{client_cert_opts, fun() -> COpts end},
+                   {server_cert_opts, fun() -> SOpts end} |
                    lists:delete(server_cert_opts,
                                 lists:delete(client_cert_opts, Config))])];
 do_init_per_group(Alg, Config) when Alg == rsa_pss_rsae;
@@ -319,8 +339,8 @@ do_init_per_group(Alg, Config) when Alg == rsa_pss_rsae;
              {extra_client, ssl_test_lib:sig_algs(Alg, Version)},
              {extra_server, ssl_test_lib:sig_algs(Alg, Version)} |
              lists:delete(cert_key_alg,
-                          [{client_cert_opts, COpts},
-                           {server_cert_opts, SOpts} |
+                          [{client_cert_opts, fun() -> COpts end},
+                           {server_cert_opts, fun() -> SOpts end} |
                            lists:delete(server_cert_opts,
                                         lists:delete(client_cert_opts, Config))])];
         false ->
@@ -340,8 +360,8 @@ do_init_per_group(Alg, Config) when Alg == rsa_pss_rsae_1_3;
               server_config := SOpts} = ssl_test_lib:make_rsa_pss_pem(rsa_alg(Alg), [], Config, ""),
             [{cert_key_alg, rsa_alg(Alg)} |
              lists:delete(cert_key_alg,
-                          [{client_cert_opts, COpts},
-                           {server_cert_opts, SOpts} |
+                          [{client_cert_opts, fun() -> COpts end},
+                           {server_cert_opts, fun() -> SOpts end} |
                            lists:delete(server_cert_opts,
                                         lists:delete(client_cert_opts, Config))])];
         false ->
@@ -354,12 +374,12 @@ do_init_per_group(Group, Config0) when Group == ecdsa;
     case lists:member(ecdsa, PKAlg) andalso (lists:member(ecdh, PKAlg) orelse lists:member(dh, PKAlg)) of
         true ->
             Config = ssl_test_lib:make_ecdsa_cert(Config0),
-            COpts = proplists:get_value(client_ecdsa_verify_opts, Config),
-            SOpts = proplists:get_value(server_ecdsa_opts, Config),
+            COpts = ssl_test_lib:ssl_options(client_ecdsa_verify_opts, Config),
+            SOpts = ssl_test_lib:ssl_options(server_ecdsa_opts, Config),
             [{cert_key_alg, ecdsa} |
              lists:delete(cert_key_alg,
-                          [{client_cert_opts, COpts},
-                           {server_cert_opts, SOpts} |
+                          [{client_cert_opts, fun() -> COpts end},
+                           {server_cert_opts, fun() -> SOpts end} |
                            lists:delete(server_cert_opts,
                                         lists:delete(client_cert_opts, Config))]
                          )];
@@ -383,8 +403,8 @@ do_init_per_group(eddsa_1_3, Config0) ->
 
             [{cert_key_alg, eddsa} |
              lists:delete(cert_key_alg,
-                          [{client_cert_opts, COpts},
-                           {server_cert_opts, SOpts} |
+                          [{client_cert_opts, fun() -> COpts end},
+                           {server_cert_opts, fun() -> SOpts end} |
                            lists:delete(server_cert_opts,
                                         lists:delete(client_cert_opts, Config0))]
                          )];
@@ -397,16 +417,22 @@ do_init_per_group(dsa = Alg, Config0) ->
     case lists:member(dss, PKAlg) andalso lists:member(dh, PKAlg) of
         true ->
             Config = ssl_test_lib:make_dsa_cert(Config0),
-            COpts = proplists:get_value(client_dsa_opts, Config),
-            SOpts = proplists:get_value(server_dsa_opts, Config),
+            COpts = ssl_test_lib:ssl_options(client_dsa_opts, Config),
+            SOpts = ssl_test_lib:ssl_options(server_dsa_opts, Config),
+            ShaDSA = case Version of
+                         {3, 3} ->
+                             [{signature_algs, [{sha, dsa}]}];
+                         _  ->
+                             []
+                     end,
             [{cert_key_alg, dsa},
              {extra_client, ssl_test_lib:sig_algs(Alg, Version) ++
-                  [{ciphers, ssl_test_lib:dsa_suites(Version)}]},
+                  [{ciphers, ssl_test_lib:dsa_suites(Version)}] ++ ShaDSA},
              {extra_server, ssl_test_lib:sig_algs(Alg, Version) ++
-                  [{ciphers, ssl_test_lib:dsa_suites(Version)}]} |
+                  [{ciphers, ssl_test_lib:dsa_suites(Version)}] ++ ShaDSA} |
              lists:delete(cert_key_alg,
-                          [{client_cert_opts, COpts},
-                           {server_cert_opts, SOpts} |
+                          [{client_cert_opts, fun() -> COpts end},
+                           {server_cert_opts, fun() -> SOpts end} |
                            lists:delete(server_cert_opts,
                                         lists:delete(client_cert_opts, Config))])];
         false ->
@@ -438,12 +464,12 @@ init_ecdsa_opts(Config0, Curve) ->
     case lists:member(ecdsa, PKAlg) andalso (lists:member(ecdh, PKAlg) orelse lists:member(dh, PKAlg)) of
         true ->
             Config = ssl_test_lib:make_rsa_ecdsa_cert(Config0, Curve),
-            COpts = proplists:get_value(client_ecdsa_verify_opts, Config),
-            SOpts = proplists:get_value(server_ecdsa_opts, Config),
+            COpts = ssl_test_lib:ssl_options(client_ecdsa_verify_opts, Config),
+            SOpts = ssl_test_lib:ssl_options(server_ecdsa_opts, Config),
             [{cert_key_alg, ecdsa} |
              lists:delete(cert_key_alg,
-                          [{client_cert_opts, ssl_test_lib:sig_algs(ecdsa, Version) ++ COpts},
-                           {server_cert_opts, ssl_test_lib:sig_algs(ecdsa, Version) ++ SOpts} |
+                          [{client_cert_opts, fun() -> ssl_test_lib:sig_algs(ecdsa, Version) ++ COpts end},
+                           {server_cert_opts, fun() -> ssl_test_lib:sig_algs(ecdsa, Version) ++ SOpts end} |
                            lists:delete(server_cert_opts,
                                         lists:delete(client_cert_opts, Config))]
                          )];
@@ -864,33 +890,14 @@ extended_key_usage_auth(Config) when is_list(Config) ->
     DefaultCertConf = ssl_test_lib:default_ecc_cert_chain_conf(proplists:get_value(name, Prop)),
     Ext = x509_test:extensions([{?'id-ce-extKeyUsage',
                                  [?'id-kp-serverAuth'], true}]),
-    #{client_config := ClientOpts0,
-      server_config := ServerOpts0} = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
-                                                                        [{server_chain, 
-                                                                          [[],[], [{extensions, Ext}]]},
-                                                                         {client_chain, DefaultCertConf}
-                                                                        ]),
-    ClientOpts = ssl_test_lib:ssl_options(extra_client, ClientOpts0, Config),
-    ServerOpts = ssl_test_lib:ssl_options(extra_server, ServerOpts0, Config),
-
-    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
-    Version = proplists:get_value(version, Config),
-    Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
-					{from, self()},
-                                        {mfa, {ssl_test_lib, send_recv_result_active, []}},
-			   {options, no_reuse(ssl_test_lib:n_version(Version)) ++ [{verify, verify_none} | ServerOpts]}]),
-    Port = ssl_test_lib:inet_port(Server),
-    Client = ssl_test_lib:start_client([{node, ClientNode}, {port, Port},
-					{host, Hostname},
-			   {from, self()},
-                                        {mfa, {ssl_test_lib, send_recv_result_active, []}},
-					{options, [{verify, verify_peer} |
-						   ClientOpts]}]),
-    
-    ssl_test_lib:check_result(Server, ok, Client, ok),
-
-    ssl_test_lib:close(Server),
-    ssl_test_lib:close(Client).
+    #{client_config := ClientOpts,
+      server_config := ServerOpts} =
+        ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                          [{server_chain,
+                                            [[],[], [{extensions, Ext}]]},
+                                           {client_chain, DefaultCertConf}
+                                          ]),
+    positive_extended_keyusage(ClientOpts, ServerOpts, Config).
 
 %%--------------------------------------------------------------------
 extended_key_usage_client_auth() ->
@@ -901,30 +908,134 @@ extended_key_usage_client_auth(Config) when is_list(Config) ->
                                        [?'id-kp-serverAuth'], true}]),
     ClientExt = x509_test:extensions([{?'id-ce-extKeyUsage',
                                        [?'id-kp-clientAuth'], true}]),
-    #{client_config := ClientOpts0,
-      server_config := ServerOpts0} = ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
-                                                                        [{client_chain, [[],[],[{extensions, ClientExt}]]},
-                                                                         {server_chain, [[],[],[{extensions, ServerExt}]]}]),
-    ClientOpts = ssl_test_lib:ssl_options(extra_client, ClientOpts0, Config),
-    ServerOpts = ssl_test_lib:ssl_options(extra_server, ServerOpts0, Config),
-   
-    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
-    Version = proplists:get_value(version, Config),
-    Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
-					{from, self()},
-                                        {mfa, {ssl_test_lib, send_recv_result_active, []}},
-                                        {options, no_reuse(ssl_test_lib:n_version(Version)) ++ [{verify, verify_peer} | ServerOpts]}]),
-    Port = ssl_test_lib:inet_port(Server),
-    Client = ssl_test_lib:start_client([{node, ClientNode}, {port, Port},
-					{host, Hostname},
-			   {from, self()},
-			   {mfa, {ssl_test_lib, send_recv_result_active, []}},
-					{options, [{verify, verify_peer} | ClientOpts]}]),
-    
-    ssl_test_lib:check_result(Server, ok, Client, ok),
+    #{client_config := ClientOpts,
+      server_config := ServerOpts} =
+        ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                          [{client_chain, [[],[],[{extensions, ClientExt}]]},
+                                           {server_chain, [[],[],[{extensions, ServerExt}]]}]),
 
-    ssl_test_lib:close(Server),
-    ssl_test_lib:close(Client).
+    positive_extended_keyusage(ClientOpts, ServerOpts, Config).
+   
+%%--------------------------------------------------------------------
+extended_key_usage_mixup_server() ->
+    [{doc,"Test cert extended_key_usage extension is always verified by having server use client extension"}].
+
+extended_key_usage_mixup_server(Config) when is_list(Config) ->
+    ClientExt = x509_test:extensions([{?'id-ce-extKeyUsage',
+                                       [?'id-kp-clientAuth'], false}]),
+    #{client_config := ClientOpts,
+      server_config := ServerOpts} =
+        ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                          [{client_chain, [[],[],[{extensions, ClientExt}]]},
+                                           {server_chain, [[],[],[{extensions, ClientExt}]]}]),
+
+    ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                      [{client_chain, [[],[],[{extensions, ClientExt}]]},
+                                       {server_chain, [[],[],[{extensions, ClientExt}]]}]),
+    
+
+    negative_extended_keyusage(ClientOpts, ServerOpts, Config).
+
+%%--------------------------------------------------------------------
+
+extended_key_usage_mixup_client() ->
+    [{doc,"Test cert extended_key_usage extension is always verified by having client use server extension"}].
+
+extended_key_usage_mixup_client(Config) when is_list(Config) ->
+    ServerExt = x509_test:extensions([{?'id-ce-extKeyUsage',
+                                       [?'id-kp-serverAuth'], false}]),
+
+    #{client_config := ClientOpts,
+      server_config := ServerOpts} =
+        ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                          [{client_chain, [[],[],[{extensions, ServerExt}]]},
+                                           {server_chain, [[],[],[{extensions, ServerExt}]]}]),
+    negative_extended_keyusage(ClientOpts, ServerOpts, Config).
+
+%%--------------------------------------------------------------------
+extended_key_usage_ca() ->
+    [{doc,"Test extended key usage in CA cert"}].
+
+extended_key_usage_ca(Config) when is_list(Config) ->
+    ServerExt = x509_test:extensions([{?'id-ce-extKeyUsage',
+                                       [?'id-kp-serverAuth'], true}]),
+    ClientExt = x509_test:extensions([{?'id-ce-extKeyUsage',
+                                       [?'id-kp-clientAuth'], true}]),
+    CAExt = x509_test:extensions([{?'id-ce-keyUsage',
+                                   [keyCertSign, digitalSignature], true}]),
+    
+    #{client_config := ClientOpts0,
+      server_config := ServerOpts0} =
+        ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                          [{client_chain, [[],[{extensions, CAExt ++ ClientExt}],
+                                                           [{extensions, ClientExt}]]},
+                                           {server_chain, [[],[{extensions, CAExt ++ ServerExt}],
+                                                           [{extensions, ServerExt}]]}]),
+
+
+    positive_extended_keyusage(ClientOpts0, ServerOpts0, Config),
+
+    CAExt1 =  x509_test:extensions([{?'id-ce-extKeyUsage',
+                                    [?'id-kp-OCSPSigning', ?'id-kp-serverAuth'], true}]),
+
+    #{client_config := ClientOpts1,
+      server_config := ServerOpts1} =
+        ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                          [{client_chain, [[],[{extensions, CAExt ++ ClientExt}],
+                                                           [{extensions, ClientExt}]]},
+                                           {server_chain, [[],[{extensions, CAExt ++ CAExt1}],
+                                                           [{extensions, ServerExt}]]}]),
+
+    positive_extended_keyusage(ClientOpts1, ServerOpts1, Config).
+
+%%--------------------------------------------------------------------
+extended_key_usage_ca_invalid() ->
+    [{doc,"Test extended key usage in CA cert that will not validate"}].
+
+extended_key_usage_ca_invalid(Config) when is_list(Config) ->
+    ServerExt = x509_test:extensions([{?'id-ce-extKeyUsage',
+                                       [?'id-kp-serverAuth'], true}]),
+    ClientExt = x509_test:extensions([{?'id-ce-extKeyUsage',
+                                       [?'id-kp-clientAuth'], true}]),
+    CAExt =  x509_test:extensions([{?'id-ce-keyUsage', [keyCertSign, cRLSign], false},
+                                   {?'id-ce-extKeyUsage',
+                                    [?'id-kp-OCSPSigning'], true}]),
+
+    #{client_config := ClientOpts,
+      server_config := ServerOpts} =
+        ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                          [{client_chain, [[],[{extensions, ClientExt}],
+                                                           [{extensions, ClientExt}]]},
+                                           {server_chain, [[],[{extensions, CAExt ++ ServerExt}],
+                                                           [{extensions, ServerExt}]]}]),
+
+    negative_extended_keyusage(ClientOpts, ServerOpts, Config).
+
+%%--------------------------------------------------------------------
+
+extended_key_usage_ca_any() ->
+    [{doc,"Test extended key usage in CA cert"}].
+
+extended_key_usage_ca_any(Config) when is_list(Config) ->
+    ServerExt = x509_test:extensions([{?'id-ce-extKeyUsage',
+                                       [?'id-kp-serverAuth'], true}]),
+    ClientExt = x509_test:extensions([{?'id-ce-extKeyUsage',
+                                       [?'id-kp-clientAuth'], true}]),
+    CAExt =  x509_test:extensions([{?'id-ce-extKeyUsage',
+                                    [?anyExtendedKeyUsage, ?'id-kp-OCSPSigning'], false}]),
+
+    #{client_config := ClientOpts,
+      server_config := ServerOpts} =
+        ssl_test_lib:make_cert_chains_der(proplists:get_value(cert_key_alg, Config),
+                                          [{client_chain, [[],[],
+                                                           [{extensions, ClientExt}]]},
+                                           {server_chain, [[],[{extensions, CAExt}],
+                                                           [{extensions, ServerExt}]]}]),
+
+    positive_extended_keyusage([{allow_any_ca_purpose, true} | ClientOpts],
+                               ServerOpts, Config),
+
+    negative_extended_keyusage(ClientOpts, ServerOpts, Config).
 
 %%--------------------------------------------------------------------
 cert_expired() ->
@@ -1021,8 +1132,7 @@ key_auth_ext_sign_only(Config) when is_list(Config) ->
 cert_auth_in_first_ca() ->
     [{doc,"Test cert auth will be available in first ca in chain, make it happen by only having one"}].
 cert_auth_in_first_ca(Config) when is_list(Config) ->
-    #{server_config := ServerOpts0,
-      client_config := ClientOpts0} =
+    #{} =
         public_key:pkix_test_data(#{server_chain => #{root => [{key, ssl_test_lib:hardcode_rsa_key(1)}],
                                                       intermediates => [[]],
                                                       peer => [{key, ssl_test_lib:hardcode_rsa_key(5)}]},
@@ -1209,13 +1319,15 @@ custom_groups(Config) ->
 %% of CertificateRequest does not contain the algorithm of the client certificate).
 %% ssl client sends an empty certificate.
 unsupported_sign_algo_cert_client_auth() ->
-     [{doc,"TLS 1.3 (backported to TLS-1.2) : Test client authentication with unsupported signature_algorithm_cert"}].
+     [{doc,"TLS 1.3 (backported to TLS-1.2) : Test client authentication with unsupported "
+       "signature_algorithm_cert"}].
 
 unsupported_sign_algo_cert_client_auth(Config) ->
     ClientOpts = ssl_test_lib:ssl_options(client_cert_opts, Config),
     ServerOpts0 = ssl_test_lib:ssl_options(server_cert_opts, Config),
     ServerOpts = [{verify, verify_peer},
-                  {signature_algs, [rsa_pkcs1_sha256, rsa_pkcs1_sha384, rsa_pss_rsae_sha256, rsa_pss_pss_sha256]},
+                  {signature_algs,
+                   [rsa_pkcs1_sha256, rsa_pkcs1_sha384, rsa_pss_rsae_sha256, rsa_pss_pss_sha256]},
                   %% Skip rsa_pkcs1_sha256!
                   {signature_algs_cert, [rsa_pkcs1_sha384, rsa_pkcs1_sha512]},
                   {fail_if_no_peer_cert, true}|ServerOpts0],
@@ -1224,7 +1336,7 @@ unsupported_sign_algo_cert_client_auth(Config) ->
         'tlsv1.3' ->
             ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, certificate_required);
         _  ->
-            ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, insufficient_security)
+            ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, unsupported_certificate)
     end.
 
 %%--------------------------------------------------------------------
@@ -1398,6 +1510,19 @@ server_certificate_authorities_disabled(Config) ->
     ssl_test_lib:basic_alert(ClientOpts, ServerOpts, Config, certificate_required),
     ssl_test_lib:basic_test(ClientOpts, [{certificate_authorities, false} | ServerOpts], Config).
 
+%%--------------------------------------------------------------------
+legacy_server_certificate_authorities_disabled() ->
+     [{doc,"Test that code pre TLS-1.3 can send an empty list for certificate authorities in the certificate request"
+       "will be run and not fail, black box verification is not possible without strict legacy client, but code coverage will show that right thing happens"}].
+
+legacy_server_certificate_authorities_disabled(Config) ->
+    Version = proplists:get_value(version,Config),
+    ClientOpts = ssl_test_lib:ssl_options(client_cert_opts, Config),
+    ServerOpts = ssl_test_lib:ssl_options(server_cert_opts, Config),
+    ssl_test_lib:basic_test([{versions, [Version]} | ClientOpts], [{versions, [Version]}, {verify, verify_peer},
+                                                                  {fail_if_no_peer_cert, true},
+                                                                  {certificate_authorities, false} | ServerOpts], Config).
+
 %%--------------------------------------------------------------------
 %% Internal functions  -----------------------------------------------
 %%--------------------------------------------------------------------
@@ -1419,3 +1544,48 @@ chain_and_root(Config) ->
     {ok, Root, Chain} = ssl_certificate:certificate_chain(OwnCert, ets:new(foo, []), ExtractedCAs, [], encoded),
     {Chain, Root}.
 
+
+positive_extended_keyusage(ClientOpts0, ServerOpts0, Config) ->
+    ClientOpts = ssl_test_lib:ssl_options(extra_client, ClientOpts0, Config),
+    ServerOpts = ssl_test_lib:ssl_options(extra_server, ServerOpts0, Config),
+
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    Version = proplists:get_value(version, Config),
+    Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
+					{from, self()},
+                                        {mfa, {ssl_test_lib, send_recv_result_active, []}},
+                                        {options, no_reuse(ssl_test_lib:n_version(Version)) ++
+                                             [{verify, verify_peer} | ServerOpts]}]),
+    Port = ssl_test_lib:inet_port(Server),
+    Client = ssl_test_lib:start_client([{node, ClientNode}, {port, Port},
+					{host, Hostname},
+                                        {from, self()},
+                                        {mfa, {ssl_test_lib, send_recv_result_active, []}},
+					{options, [{verify, verify_peer} | ClientOpts]}]),
+
+    ssl_test_lib:check_result(Server, ok, Client, ok),
+
+    ssl_test_lib:close(Server),
+    ssl_test_lib:close(Client).
+
+negative_extended_keyusage(ClientOpts0, ServerOpts0, Config) ->
+    ClientOpts = ssl_test_lib:ssl_options(extra_client, ClientOpts0, Config),
+    ServerOpts = ssl_test_lib:ssl_options(extra_server, ServerOpts0, Config),
+
+    {ClientNode, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+    Version = proplists:get_value(version, Config),
+    Server = ssl_test_lib:start_server_error([{node, ServerNode}, {port, 0},
+                                              {from, self()},
+                                              {mfa, {ssl_test_lib, no_result, []}},
+                                              {options, [{verify, verify_peer},
+                                                         {fail_if_no_peer_cert, true}] ++
+                                                   no_reuse(ssl_test_lib:n_version(Version)) ++ ServerOpts
+                                              }]),
+    Port = ssl_test_lib:inet_port(Server),
+    Client = ssl_test_lib:start_client_error([{node, ClientNode}, {port, Port},
+                                              {host, Hostname},
+                                              {from, self()},
+                                              {mfa, {ssl_test_lib, no_result, []}},
+                                              {options, [{verify, verify_peer} | ClientOpts]}]),
+
+    ssl_test_lib:check_server_alert(Server, Client, unsupported_certificate).
